<?php

namespace Drupal\warden\Controller;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\PageCache\ResponsePolicy\KillSwitch;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\warden\Service\WardenManager;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Default controller for the warden module.
 */
class DefaultController extends ControllerBase {
  use StringTranslationTrait;

  /**
   * Warden manager service.
   *
   * @var \Drupal\warden\Service\WardenManager
   */
  protected $wardenManager;

  /**
   * Config Factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The Messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Logger Factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * RequestStack service.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * The kill switch.
   *
   * @var \Drupal\Core\PageCache\ResponsePolicy\KillSwitch
   */
  protected $killSwitch;

  /**
   * {@inheritdoc}
   */
  public function __construct(WardenManager $wardenManager, ConfigFactoryInterface $configFactory, MessengerInterface $messenger, LoggerChannelFactoryInterface $loggerFactory, RequestStack $requestStack, TimeInterface $time, KillSwitch $killSwitch) {
    $this->wardenManager = $wardenManager;
    $this->configFactory = $configFactory;
    $this->messenger = $messenger;
    $this->loggerFactory = $loggerFactory->get('warden');
    $this->requestStack = $requestStack;
    $this->time = $time;
    $this->killSwitch = $killSwitch;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('warden.manager'),
      $container->get('config.factory'),
      $container->get('messenger'),
      $container->get('logger.factory'),
      $container->get('request_stack'),
      $container->get('datetime.time'),
      $container->get('page_cache_kill_switch')
    );
  }

  /**
   * @return \Drupal\Core\Config\Config
   *   Warden configuration.
   */
  public function getWardenConfig() {
    return $this->configFactory->get('warden.settings');
  }

  /**
   * @return \Drupal\warden\Service\WardenManager
   *   Warden Manager service.
   */
  public function getWardenManager() {
    return $this->wardenManager;
  }

  /**
   * Warden registration form.
   *
   * @return array
   *   Render array for the registration markup.
   */
  public function wardenRegistration() {
    $build = [];

    $warden_path = $this->getWardenConfig()->get('warden_server_host_path');

    if (empty($warden_path)) {
      $this->messenger->addMessage($this->t('You are missing some Warden configuration. Please read the README file for more details.'), MessengerInterface::TYPE_ERROR);
      return [];
    }

    $build['local_token'] = [
      '#markup' => '<p>' . $this->t('Site security token: %token', [
        '%token' => $this->getWardenManager()->getLocalToken(),
      ]) . '</p>',
    ];

    $build['message'] = [
      '#markup' => '<p>' . $this->t('To add this site to your Warden dashboard click the button below. Read the README file which comes with the module for configuration information.') . '</p>',
    ];

    $build['button'] = [
      '#markup' => $this->t('<a class="button" href="@url">Add this site to your Warden Dashboard</a>', [
        '@url' => $this->generateWardenRegistrationRedirect(),
      ]),
    ];

    return $build;
  }

  /**
   * Generate a redirect to the Warden server for site registration.
   *
   * @return string
   *   Returns the url of the Warden server.
   * @throws \Exception
   * @throws \WardenApi\Exception\EncryptionException
   */
  protected function generateWardenRegistrationRedirect() {
    global $base_url;
    $site_url = $base_url;
    $site_url .= "|" . $this->getWardenManager()->getLocalToken();
    $site_url_encrypted = $this->getWardenManager()->encrypt($site_url);
    $site_host_path = $this->getWardenConfig()->get('warden_server_host_path');
    return $site_host_path . '/sites/add?data=' . $site_url_encrypted;
  }

  /**
   * Access control to ensure authorised requests to get system data.
   *
   * @return \Drupal\Core\Access\AccessResult
   *   Returns the access result.
   * @throws \Exception
   */
  public function wardenAccess() {
    $allow_requests = $this->getWardenConfig()->get('warden_allow_requests');

    if (empty($allow_requests)) {
      $this->loggerFactory->warning('Update request denied: warden_allow_requests is set to FALSE', []);
      return AccessResult::forbidden();
    }

    $token = $this->requestStack->getCurrentRequest()->request->get('token');
    if (empty($token)) {
      $this->loggerFactory->warning('Update request denied: request body is empty or missing the security token', []);
      return AccessResult::forbidden();
    }

    $time = $this->time->getRequestTime();
    if (!$this->getWardenManager()->isValidWardenToken($token, $time)) {
      $this->loggerFactory->warning('Update request denied: Failed to validate security token in request at timestamp @time', [
        '@time' => $time,
      ]);
      return AccessResult::forbidden();
    }

    $allowed_ips = $this->getWardenConfig()->get('warden_public_allow_ips');

    if (!empty($allowed_ips)) {
      $ip_address = $this->requestStack->getCurrentRequest()->getClientIp();
      $allowed_ips = explode(',', $this->getWardenConfig()->get('warden_public_allow_ips'));

      foreach ($allowed_ips as &$address) {
        if ($ip_address === $address) {
          return AccessResult::allowed();
        }
      }

      // No IP addresses match.
      $this->loggerFactory->warning('Update request denied: The requesting IP is not in the warden_public_allow_ips whitelist - @ip', [
        '@ip' => $ip_address,
      ]);
      return AccessResult::forbidden();
    }

    return AccessResult::allowed();
  }

  /**
   * Routing callback to retrieve the data stored on the site.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   JSON response on the Warden update status.
   */
  public function wardenStatus() {
    $this->killSwitch->trigger();
    $response = new JsonResponse();

    try {
      $this->getWardenManager()->updateWarden();
      $response->setData(['data' => 'OK']);
    }
    catch (\Exception $e) {
      $this->loggerFactory->error('Warden Status error', $e);
      $response->setStatusCode(500);
      $response->setData(['error' => 'Internal fault']);
    }

    return $response;
  }

}
