<?php

namespace Drupal\Tests\warden\Unit\Service;

use Drupal\Tests\UnitTestCase;
use Drupal\warden\Service\WardenManager;

/**
 * @coversDefaultClass \Drupal\warden\Service\WardenManager
 * @group warden
 */
class WardenManagerTest extends UnitTestCase {

  /**
   * @var \Drupal\warden\Service\WardenManager
   */
  protected $wardenManager;

  /**
   * @var string
   */
  protected $token;

  /**
   * @var \Drupal\Core\Extension\InfoParser|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $infoParser;

  /**
   * @var \Drupal\Core\Config\ConfigFactory|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $configFactory;

  /**
   * @var \Drupal\Core\File\FileSystemInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  private $fileSystem;

  /**
   * @var \Drupal\Core\Extension\ThemeHandlerInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  private $themeHandler;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->token = hash('sha256', 42);

    $this->infoParser = $this->createMock('Drupal\Core\Extension\InfoParser', [], [], '', FALSE);
    $this->configFactory = $this->createMock('Drupal\Core\Config\ConfigFactory', [], [], '', FALSE);
    $this->fileSystem = $this->createMock('Drupal\Core\File\FileSystemInterface', [], [], '', FALSE);
    $this->themeHandler = $this->createMock('Drupal\Core\Extension\ThemeHandlerInterface', [], [], '', FALSE);

    $this->expectConfigFactoryAccess();

    $this->wardenManager = new WardenManager($this->configFactory, $this->fileSystem, $this->themeHandler);
    $this->wardenManager
      ->setBaseUrl('http://www.example.com')
      ->setTime(12345678)
      ->setInfoParser($this->infoParser)
      ->setThemes([])
      ->setModules([]);
  }

  /**
   * Expect access to the config factory via the WardenManager constructor.
   */
  protected function expectConfigFactoryAccess() {
    $wardenConfig = $this->createMock('Drupal\Core\Config\Config', [], [], '', FALSE);
    $systemConfig = $this->createMock('Drupal\Core\Config\Config', [], [], '', FALSE);

    $this->configFactory->expects($this->any())
      ->method('get')
      ->willReturnCallback(function ($setting_name) use ($wardenConfig, $systemConfig) {
        $settings = [
          'warden.settings' => $wardenConfig,
          'system.site' => $systemConfig,
        ];
        return $settings[$setting_name];
      }
      );

    $systemConfig->expects($this->once())
      ->method('get')
      ->with('name')
      ->willReturn('My Website');

    $wardenConfig->expects($this->any())
      ->method('get')
      ->willReturnCallback(function ($setting_name) {
        $settings = [
          'warden_preg_match_contrib' => '{^modules\/contrib\/*}',
          'warden_preg_match_custom' => '{^modules\/custom\/*}',
          'warden_match_custom' => TRUE,
          'warden_match_contrib' => TRUE,
          'warden_match_library' => FALSE,
          'warden_token' => $this->token,
          'warden_server_host_path' => 'http://warden.example.com',
          'warden_http_username' => '',
          'warden_http_password' => '',
          'warden_certificate_path' => '',
          'warden_list_libraries' => NULL,
        ];
        return $settings[$setting_name];
      }
      );
  }

  /**
   * @param string $name
   *   Module name to mock.
   * @param string $type
   *   Module type to mock.
   * @return \Drupal\Core\Extension\Extension|\PHPUnit\Framework\MockObject\MockObject
   *   Returns a module mock.
   */
  protected function getMockExtension($name, $type = 'contrib') {
    /** @var \Drupal\Core\Extension\Extension|\PHPUnit\Framework\MockObject\MockObject $module_b */
    $extension = $this->createMock('Drupal\Core\Extension\Extension', [], [], '', FALSE);
    $extension->expects($this->once())
      ->method('getPathname')
      ->willReturn("modules/{$type}/{$name}/{$name}.info.yml");

    return $extension;
  }

  /**
   * @param array $moduleNames
   *   A list of module names (e.g. ['module_a', 'module_b']
   *   The info parser will expect there to be 2 calls to parse
   *   and it will return an array each time of the form
   *   ['version' => '8.x-1.0'].
   */
  protected function expectContribModules(array $moduleNames) {
    $results = [];

    foreach ($moduleNames as $moduleName) {
      $results["modules/contrib/{$moduleName}/{$moduleName}.info.yml"] = [
        'version' => '8.x-1.0',
      ];
    }

    $this->infoParser->expects($this->exactly(count($moduleNames)))
      ->method('parse')
      ->willReturnCallback(function ($info_filename) use ($results) {
          return $results[$info_filename];
      }
      );
  }

  /**
   * Tests generating module data with no modules.
   */
  public function testGenerateDataNoModules() {
    $expected_data = [
      'core' => [
        'drupal' => [
          'version' => \Drupal::VERSION,
        ],
      ],
      'contrib' => [],
      'custom' => [],
      'url' => 'http://www.example.com',
      'site_name' => 'My Website',
      'key' => $this->token,
      'time' => 12345678,
      'library' => 'disabled',
    ];

    $actual_data = $this->wardenManager->generateSiteData();
    $this->assertEquals($expected_data, $actual_data);
  }

  /**
   * Tests generating module data with one module.
   */
  public function testGenerateDataOneModule() {
    $expected_data = [
      'core' => [
        'drupal' => [
          'version' => \Drupal::VERSION,
        ],
      ],
      'contrib' => [
        'Module A' => [
          'version' => '8.x-1.0',
        ],
      ],
      'custom' => [],
      'url' => 'http://www.example.com',
      'site_name' => 'My Website',
      'key' => $this->token,
      'time' => 12345678,
      'library' => 'disabled',
    ];

    $this->wardenManager->setModules([
      'module_a' => $this->getMockExtension('module_a'),
    ]);

    $this->infoParser->expects($this->once())
      ->method('parse')
      ->with('modules/contrib/module_a/module_a.info.yml')
      ->willReturn([
        'project' => 'Module A',
        'version' => '8.x-1.0',
      ]);

    $actual_data = $this->wardenManager->generateSiteData();
    $this->assertEquals($expected_data, $actual_data);
  }

  /**
   * Tests generating module data with two modules.
   */
  public function testGenerateDataTwoModules() {
    $expected_data = [
      'core' => [
        'drupal' => [
          'version' => \Drupal::VERSION,
        ],
      ],
      'contrib' => [
        'module_a' => [
          'version' => '8.x-1.0',
        ],
        'module_b' => [
          'version' => '8.x-1.0',
        ],
      ],
      'custom' => [],
      'url' => 'http://www.example.com',
      'site_name' => 'My Website',
      'key' => $this->token,
      'time' => 12345678,
      'library' => 'disabled',
    ];

    $this->wardenManager->setModules([
      'module_a' => $this->getMockExtension('module_a'),
      'module_b' => $this->getMockExtension('module_b'),
    ]);

    $this->expectContribModules(['module_a', 'module_b']);

    $actual_data = $this->wardenManager->generateSiteData();
    $this->assertEquals($expected_data, $actual_data);
  }

}
